#include <stdlib.h>
#include <string.h>
#include <system/api.h>
#include "gblib.h"
#include "kmenu.h"

#define GET_ITEM(m,i) (m->items[i])
#define GET_FOCUSED_ITEM(m) (m->items[m->focus_idx])

// TODO
static const size_t font_height = 9;
static const size_t font_width = 6;

static t_system_colors *color_table = NULL;

void KMENU_API kmenu_init(t_system_colors *colors)
{
	color_table = colors;
	
	size_t old_mask = kos_SetEventMask(0);
	old_mask |= ( 1 << (SYSEVENT_IPC-1));
	kos_SetEventMask(old_mask);
}

static int ksubmenu_dispatch_event_ex(ksubmenu_t *menu, int event);
static int ksubmenu_dispatch_cursorevent(ksubmenu_t *menu);
static int ksubmenu_dispatch_btnpress(ksubmenu_t *menu, int id);

kmenu_t * KMENU_API kmenu_new(void)
{
	kmenu_t *menu = malloc(sizeof(*menu));
	if (!menu)
		return NULL;
	memset(menu, 0, sizeof(*menu));
	
	menu->focus_idx    = -1;
	menu->submenu_tid  = -1;
	return menu;
}

void KMENU_API kmenu_delete(kmenu_t *menu)
{
	if (!menu)
		return;
}

void KMENU_API kmenu_dispatch_event(ksubmenu_t *menu, int event)
{
	if (!menu)
		return;
	ksubmenu_dispatch_event_ex(menu, event);
}

void KMENU_API kmainmenu_dispatch_cursorevent(ksubmenu_t *menu)
{
	if (!menu)
		return;
	ksubmenu_dispatch_cursorevent(menu);
}

void KMENU_API kmainmenu_dispatch_btnpress(ksubmenu_t *menu, int id)
{
	if (!menu)
		return;
	ksubmenu_dispatch_btnpress(menu, id);
}

void KMENU_API kmenu_add(kmenu_t *menu, kmenuitem_t *submenu)
{
	if (!menu)
		return;
}

static void __stdcall kmenu_paint(ksubmenu_t *menu)
{
	if (!menu)
		return;
	t_thread_info *ti = kos_GetThreadInfo(-1);
	if (!ti)
		return;

	size_t old_mask = kos_SetEventMask(0);
	old_mask &= 0x3FFFFFFF;
	//old_mask |= 0x80000000;
	old_mask |= (1 << (SYSEVENT_MOUSE - 1));
	kos_SetEventMask(old_mask);
	
	menu->left  = 0;//ti->client_area_size.left;
	menu->top   = 0;//ti->client_area_size.top;
	menu->width = ti->window_size.width - KMENU_LBORDER_SIZE - KMENU_DBORDER_SIZE - 4*2; // -borderwidth*2
	//menu->width = ti->client_area_size.width - KMENU_LBORDER_SIZE - KMENU_DBORDER_SIZE - 4*2; // -borderwidth*2
	
	// get preffered size for menu list
	size_t pref_width = 0;
	size_t pref_height = 0;
	size_t line_height = 0;
	
	
	for (size_t i = 0; i < menu->count; i++)
	{
		kmenuitem_t *it = GET_ITEM(menu, i);
		//it->is_focused = 0;
		it->style = KMENUITEM_MAINMENU;
		if (!it->is_visible || it->type == KMENUITEM_SEPARATOR)
			continue;
		
		it->left = menu->left + pref_width;
		it->top  = menu->top  + pref_height + KMENU_LBORDER_SIZE;
		
		size_t item_width  = kmenuitem_get_preffered_width(it);
		size_t item_height = kmenuitem_get_preffered_height(it);
		if (item_height > line_height)
			line_height = item_height;

		pref_width += item_width;
		if (pref_width >= menu->width)
		{
			pref_width  = 0;
			pref_height += line_height;
			pref_height += KMENU_LBORDER_SIZE + KMENU_DBORDER_SIZE + KMENU_LBORDER_SIZE;
			line_height = item_height;
			
			it->left = menu->left + pref_width;
			it->top  = menu->top  + pref_height + KMENU_LBORDER_SIZE;
			
			pref_width  += item_width;
				
			kos_DrawLine(menu->left, 
				menu->top + pref_height, 
				menu->left + menu->width, 
				menu->top + pref_height, 
				color_table->_3d_dark);
			kos_DrawLine(menu->left, 
				menu->top + pref_height+1, 
				menu->left + menu->width, 
				menu->top + pref_height+1, 
				color_table->_3d_light);
			
		}
		
		t_rect rc_items = {0};
		rc_items.top    = it->top;
		rc_items.left   = it->left;
		rc_items.right  = it->left + menu->width;
		rc_items.bottom = it->top  + pref_height;
		
		kmenuitem_draw(it, &rc_items);
	}
	line_height += KMENU_LBORDER_SIZE + KMENU_DBORDER_SIZE;
	menu->height = pref_height + line_height;
	
	// top and bottom
	kos_DrawLine(menu->left, 
				menu->top, 
				menu->left + menu->width, 
				menu->top, 
				color_table->_3d_light);
	kos_DrawLine(menu->left, menu->top + menu->height, menu->left + menu->width, menu->top + menu->height, color_table->_3d_dark);
	
	
	
	kos_HeapFree(ti);
}

void KMENU_API kmainmenu_draw(ksubmenu_t *menu)
{
	if (!menu)
		return;
	menu->paint = kmenu_paint;
	menu->self_wnd = kos_GetForegroundWindow();
	
	kmenu_paint(menu);
}

// ----------------------------------------------------------

#define EVENTLOOP_STOP 0
#define EVENTLOOP_CONTINUE 1

static int ksubmenu_draw_sub(ksubmenu_t *menu, t_point *pt, int level);
static void __stdcall ksubmenu_paint(ksubmenu_t *menu);

ksubmenu_t * KMENU_API ksubmenu_new(void)
{
	ksubmenu_t *menu = malloc(sizeof(*menu));
	if (!menu)
		return NULL;
	memset(menu, 0, sizeof(*menu));
	menu->focus_idx    = -1;
	menu->return_btnid = -1;
	menu->submenu_tid  = -1;
	menu->paint = ksubmenu_paint;
	return menu;
}

void KMENU_API ksubmenu_delete(ksubmenu_t *menu)
{
	if (!menu)
		return;
	if (menu->items)
	{
		for (size_t i = 0; i < menu->count; i++)
		{
			kmenuitem_delete(GET_ITEM(menu, i));
		}
	}
	free(menu);
}

static void ksubmenu_paint_item(ksubmenu_t *menu, int idx)
{
	if (!menu || (idx > menu->count) || (idx == -1))
		return;
	
	kmenuitem_t *it = GET_ITEM(menu, idx);

	t_rect rc_items = {0};
	/*rc_items.left   = KMENU_LBORDER_SIZE;
	rc_items.right  = menu->width - KMENU_LBORDER_SIZE - KMENU_DBORDER_SIZE;
	rc_items.top    = it->top;
	rc_items.bottom = rc_items.top + it->height;*/
	rc_items.left   = it->left;
	rc_items.right  = it->left + it->width;
	rc_items.top    = it->top;
	rc_items.bottom = it->top + it->height;
	kmenuitem_draw(it, &rc_items);
}

static void __stdcall ksubmenu_paint(ksubmenu_t *menu)
{
	if (!menu)
		return;
	/*if (menu->is_opened)
		return;*/
	
	kos_WindowRedrawStart();
	
	kos_DefineAndDrawWindow(menu->left, menu->top, menu->width, menu->height, 0x61000000 | color_table->btn_face, 0x01000000, 0);
	// draw border
	kos_DrawLine(0,            0,             menu->width, 0,            color_table->win_inframe);
	kos_DrawLine(0,            menu->height,  menu->width, menu->height, color_table->win_inframe);	
	kos_DrawLine(0,            0,             0,           menu->height, color_table->win_inframe);
	kos_DrawLine(menu->width,  0,             menu->width, menu->height, color_table->win_inframe);
	
	kos_DrawLine(1,            1,             menu->width-1, 1,              color_table->_3d_light);
	kos_DrawLine(1,            1,             1,             menu->height-1, color_table->_3d_light);
	kos_DrawLine(menu->width-1,1,             menu->width-1, menu->height-1, color_table->win_body);
	kos_DrawLine(1,            menu->height-1,menu->width-1, menu->height-1, color_table->win_body);
	
	// draw and calc items size
	t_rect rc_items = {0};
	rc_items.top   = KMENU_LBORDER_SIZE;
	rc_items.left  = KMENU_LBORDER_SIZE;
	rc_items.right = menu->width - KMENU_LBORDER_SIZE - KMENU_DBORDER_SIZE;
	for (size_t i = 0; i < menu->count; i++)
	{
		kmenuitem_t *it = GET_ITEM(menu, i);
		if (!it->is_visible)
			continue;
		rc_items.bottom = rc_items.top  + it->height;
		it->left  = rc_items.left;
		it->top   = rc_items.top;
		it->width = rc_items.right; //menu->width - KMENU_BORDER_SIZE;//*2;
		kmenuitem_draw(it, &rc_items);
		rc_items.top += it->height;
	}
	
	kos_WindowRedrawFinish();
}

static void ksubmenu_remove_focus(ksubmenu_t *menu)
{
	if (!menu)
		return;
	
	if (menu->focus_idx == -1)
		return;
	
	if (menu->submenu_tid != -1)
	{
		// если было открыто submenu - закрываем
		kos_TerminateThreadById(menu->submenu_tid);
		menu->submenu_tid = -1;
	}
	
	kmenuitem_t *it = GET_FOCUSED_ITEM(menu);
	it->is_focused  = 0;
	ksubmenu_paint_item(menu, menu->focus_idx);
	menu->focus_idx = -1;
}

static int ksubmenu_on_item_click(ksubmenu_t *menu);

static void ksubmenu_set_focus(ksubmenu_t *menu, size_t idx)
{
	if (!menu)
		return;
	
	if (menu->focus_idx == idx)
		return;
	
	int need_open = 0;
	if (menu->submenu_tid != -1)
		need_open = 1;
	
	ksubmenu_remove_focus(menu);
	
	kmenuitem_t *it = GET_ITEM(menu, idx);
	it->is_focused  = 1;
	ksubmenu_paint_item(menu, idx);
	menu->focus_idx = idx;
	
	if (/*!menu->is_opened && */need_open)
		ksubmenu_on_item_click(menu);
}

static void ksubmenu_move_focus(ksubmenu_t *menu, int delta)
{
	if (!menu)
		return;
	
	// is first
	if (menu->focus_idx == -1 && delta < 0)
	{
		menu->focus_idx = menu->count-1;
	}
	
	int new_idx = menu->focus_idx + delta;
	if (new_idx < 0)
		new_idx = menu->count-1;
	if (new_idx >= menu->count)
		new_idx = 0;
	
	if (delta < 0)
	{
		for (int i = new_idx; i >= 0; i--)
		{
			if (!GET_ITEM(menu, i)->is_enabled || !GET_ITEM(menu, i)->is_visible)
				continue;
			new_idx = i;
			break;
		}
	}
	else
	{
		for (int i = new_idx; i < menu->count; i++)
		{
			if (!GET_ITEM(menu, i)->is_enabled || !GET_ITEM(menu, i)->is_visible)
				continue;
			new_idx = i;
			break;
		}
	}
	
	if (menu->focus_idx >= 0)
		ksubmenu_remove_focus(menu);
	ksubmenu_set_focus(menu, new_idx);
}

static int ksubmenu_point_in_menu(ksubmenu_t *menu, t_point *pos)
{
	if (!menu || !pos)
		return 0;
	
	return ((pos->left >= menu->left) && 
	        (pos->left <= (menu->left + menu->width)) &&
	        (pos->top >= menu->top) &&
	        (pos->top <= (menu->top + menu->height)));
}

static int ksubmenu_on_item_click(ksubmenu_t *menu)
{
	if (!menu)
		return EVENTLOOP_STOP;
	if (menu->focus_idx == -1)
		return EVENTLOOP_CONTINUE;

	switch (GET_FOCUSED_ITEM(menu)->type)
	{
		case KMENUITEM_NORMAL:
			menu->return_btnid = GET_FOCUSED_ITEM(menu)->btnid;
			return EVENTLOOP_STOP;
		case KMENUITEM_SUBMENU:
		{
			kmenuitem_t *it = GET_FOCUSED_ITEM(menu);
			t_point pt = {0};
			int level = menu->level+1;
			if (menu->is_opened)
			{
				pt.left = menu->left + menu->width;
				pt.top  = menu->top  + it->top;
			}
			else
			{
				t_thread_info *ti = kos_GetThreadInfo(-1);
				if (!ti)
					return EVENTLOOP_STOP;
				pt.left = ti->window_size.left + ti->client_area_size.left + it->left;
				pt.top  = ti->window_size.top  + ti->client_area_size.top  + it->top + it->height;
				level = 0;
				kos_HeapFree(ti);
			}
			menu->submenu_tid = ksubmenu_draw_sub(GET_FOCUSED_ITEM(menu)->submenu, &pt, level);
			return EVENTLOOP_CONTINUE;
		}
	}
	return EVENTLOOP_CONTINUE;
}

static int ksubmenu_dispatch_btnpress(ksubmenu_t *menu, int id)
{
	// пришло от дочернего submenu
	//int id = kos_GetPressedButtonId();
	switch (id)
	{
		case 1:
		{
			menu->submenu_tid = -1;
			t_point pos = {0};
			kos_GetCursorPosByScreen(&pos);
			if (ksubmenu_point_in_menu(menu, &pos))
			{
				break;
			}
			menu->return_btnid = id;
			return EVENTLOOP_STOP;
		}
		default:
			// все остальные передаем меню выше
			menu->return_btnid = id;
			return EVENTLOOP_STOP;
	}
	
	return EVENTLOOP_CONTINUE;
}

static int ksubmenu_dispatch_keyevent(ksubmenu_t *menu)
{
	unsigned char ch;
	int keytype = kos_GetKey(&ch);
	switch (ch)
	{
		case VK_ESC:
			return EVENTLOOP_STOP;
			
		case VK_DOWN:
		{
			ksubmenu_move_focus(menu, 1);
			break;
		}
		case VK_UP:
		{
			ksubmenu_move_focus(menu, -1);
			break;
		}
		case VK_RETURN:
		{
			return ksubmenu_on_item_click(menu);
		}
	}
	return EVENTLOOP_CONTINUE;
}

static int ksubmenu_dispatch_cursorevent(ksubmenu_t *menu)
{
	t_point pos = {0};
	if (menu->is_opened)
	{
		kos_GetCursorPosByScreen(&pos);
	}
	else 
	{
		kos_GetCursorPosByWindow(&pos);
		/*t_thread_info *t = kos_GetThreadInfo(menu->submenu_tid);
		if (!t)
			menu->submenu_tid = -1;
		kos_HeapFree(t);*/
		if (!kos_GetSlotByTID(menu->submenu_tid))
			menu->submenu_tid = -1;
	}

	size_t mk = kos_GetMouseKeyState();
	if (mk)
	{
		// игнорируем клики мыши если есть дочернее submenu
		if (menu->submenu_tid != -1)
			return EVENTLOOP_CONTINUE;

		if (ksubmenu_point_in_menu(menu, &pos))
		{
			// клик по пункту меню
			return ksubmenu_on_item_click(menu);
		}
		else
		{
			ksubmenu_remove_focus(menu);
			// сообщаем родительскому меню о клике вне области
			menu->return_btnid = 1;
			// клик вне окна - закрываемся
			return EVENTLOOP_STOP;
		}
	}
	// проверяем попадание курсора при движении мышкой
	if (ksubmenu_point_in_menu(menu, &pos))
	{
		pos.top  -= menu->top;
		pos.left -= menu->left;
		size_t mouseover_idx = -1;
		for (size_t i = 0; i < menu->count; i++)
		{
			kmenuitem_t *it = GET_ITEM(menu, i);
			if ((pos.top >= it->top) &&
				(pos.top <= (it->top + it->height)) &&
				(pos.left >= it->left) &&
				(pos.left <= (it->left + it->width))
			)
			{
				mouseover_idx = i;
				break;
			}
		}
		
		if (mouseover_idx != -1)
		{
			// курсор над неактивным пунктом меню
			if (!GET_ITEM(menu, mouseover_idx)->is_enabled)
			{
				ksubmenu_remove_focus(menu);
				return EVENTLOOP_CONTINUE;
			}
			
			// не перерисовываем уже выделенный
			if (menu->focus_idx != mouseover_idx)
			{
				ksubmenu_set_focus(menu, mouseover_idx);
			}
		}
	}
	else
	{
		if ((!menu->is_opened) && (menu->submenu_tid == -1))
			ksubmenu_remove_focus(menu);
	}
	return EVENTLOOP_CONTINUE;
}

static int ksubmenu_dispatch_event_ex(ksubmenu_t *menu, int event)
{
	if (!menu)
		return EVENTLOOP_STOP;
	
	switch (event)
	{
		case SYSEVENT_WNDREDRAW:
		{
			//ksubmenu_paint(menu);
			menu->paint(menu);
			break;
		}
		case SYSEVENT_BTNPRESS:
		{
			return ksubmenu_dispatch_btnpress(menu, kos_GetPressedButtonId());
		}
		case SYSEVENT_KEYPRESS:
		{
			if (menu->is_opened)
				return ksubmenu_dispatch_keyevent(menu);
		}
		case SYSEVENT_MOUSE:
		{
			return ksubmenu_dispatch_cursorevent(menu);
		}
		
	}
	return EVENTLOOP_CONTINUE;
}

static void ksubmenu_thread(ksubmenu_t *menu)
{
	size_t old_mask = kos_SetEventMask(0);
	old_mask &= 0x3FFFFFFF;
	//old_mask |= 0x80000000;
	old_mask |= (1 << (SYSEVENT_MOUSE - 1));
	kos_SetEventMask(old_mask);
	
	// get preffered size for menu list
	size_t pref_width = 0;
	size_t pref_height = 0;
	
	for (size_t i = 0; i < menu->count; i++)
	{
		kmenuitem_t *it = GET_ITEM(menu, i);
		it->is_focused = 0;
		if (!it->is_visible)
			continue;
		size_t item_width = kmenuitem_get_preffered_width(it);
		if (item_width > pref_width)
			pref_width  = item_width;
		pref_height += kmenuitem_get_preffered_height(it);
	}
	menu->width  = pref_width + KMENU_LBORDER_SIZE + KMENU_DBORDER_SIZE + font_width*3; //border*2 + space for draw ->
	menu->height = pref_height + KMENU_LBORDER_SIZE + KMENU_DBORDER_SIZE; //border*2
	
	//ksubmenu_paint(menu);
	menu->paint(menu);
	menu->is_opened = 1;

	while (1)
	{
		int event = kos_WaitForEvent();
		if (!ksubmenu_dispatch_event_ex(menu, event))
			break;
	}
	kos_SetForegroundWindow(menu->parent_wnd);
	// send seleted menu id
	if (menu->return_btnid != -1)
	{
		/*while (kos_GetForegroundWindow() != menu->parent_wnd)
			kos_Sleep(100);*/
		switch (menu->return_btnid)
		{
			case 1:
				if (menu->level == 0)
					break;
				kos_SendMessage(SYSEVENT_BTNPRESS, menu->return_btnid);
				kos_ThreadYield();
				break;
			default:
				kos_SendMessage(SYSEVENT_BTNPRESS, menu->return_btnid);
				kos_ThreadYield();
				break;
		}
		//kos_IPCSend(menu->parent_tid, &menu->return_btnid, sizeof(menu->return_btnid));
	}
	
	menu->is_opened = 0;

	kos_ExitApp();
}

void KMENU_API ksubmenu_close(ksubmenu_t *menu)
{
	if (!menu)
		return;
	if (menu->is_opened)
	{
		kos_TerminateThreadById(menu->tid);
	}
}

int KMENU_API ksubmenu_draw(ksubmenu_t *menu, t_point *pt)
{
	if (!menu || !pt)
		return -1;
	if (menu->is_opened)
	{
		ksubmenu_close(menu);
	}
	
	menu->focus_idx = -1;
	menu->submenu_tid = -1;
	menu->return_btnid = -1;
	menu->level = 0;
	menu->left = pt->left;
	menu->top  = pt->top;
	menu->parent_wnd = kos_GetForegroundWindow();
	menu->parent_tid = kos_GetCurrentThreadId();
	menu->tid = kos_CreateThread(ksubmenu_thread, menu);
	kos_ThreadYield();
	return menu->tid;
}

static int ksubmenu_draw_sub(ksubmenu_t *menu, t_point *pt, int level)
{
	if (!menu || !pt)
		return -1;
	if (menu->is_opened)
	{
		ksubmenu_close(menu);
	}
	
	menu->focus_idx = -1;
	menu->submenu_tid = -1;
	menu->return_btnid = -1;
	menu->level = level;
	menu->left = pt->left;
	menu->top  = pt->top;
	menu->parent_wnd = kos_GetForegroundWindow();
	menu->parent_tid = kos_GetCurrentThreadId();
	menu->tid = kos_CreateThread(ksubmenu_thread, menu);
	kos_ThreadYield();
	return menu->tid;
}

void KMENU_API ksubmenu_add(ksubmenu_t *menu, kmenuitem_t *menuitem)
{
	if (!menu)
		return;
	// TODO need lock
	if (menu->count == 0)
	{
		menu->items = malloc(sizeof(kmenuitem_t *));
		if (!menu->items)
			return;
	}
	else
	{
		menu->items = realloc(menu->items, (menu->count+1)*sizeof(kmenuitem_t *));
	}
	GET_ITEM(menu, menu->count) = menuitem;
	menu->count++;
}


// ----------------------------------------------------------

static void __stdcall kmenuitem_default_paint(kmenuitem_t *item, t_rect *rc)
{
	if (!item || !rc)
		return;
	
	GB_BMP gb = {0};
	gb.w = item->width;
	gb.h = item->height;
	
	gb.bmp = malloc(gb.w * gb.h * 3); // 3 - rgb
	if (!gb.bmp)
		return;
	
	int bg_color = color_table->win_body; //color_table->btn_face;
	int fg_color = color_table->win_text; //color_table->btn_text;
	int br_color = color_table->btn_fcframe; // border focused
	
	if (!item->is_enabled)
	{
		bg_color = color_table->win_body; //color_table->btn_inface;
		fg_color = color_table->btn_intext;		
	}
	if (item->is_focused)
	{
		bg_color = color_table->btn_face; //color_table->btn_fcface;
		fg_color = color_table->btn_text; //color_table->btn_fctext;		
	}
	
	gb_bar(&gb, 0, 0, item->width, item->height, bg_color);
	
	switch (item->type)
	{
		case KMENUITEM_NORMAL:
		case KMENUITEM_SUBMENU:
		{
			if (item->is_focused)
			{
				//gb_rect(&gb, 0, 0, item->width, item->height, br_color);
				gb_line(&gb, 0, 0, item->width, 0, color_table->_3d_dark);
				gb_line(&gb, 0, 0, 0, item->height-1, color_table->_3d_dark);
				gb_line(&gb, 0, item->height-1, item->width, item->height-1, color_table->_3d_light);
				gb_line(&gb, item->width-1, 0, item->width-1, item->height-1, color_table->_3d_light);
				//gb_line(&gb, 0, 0, item->width, 0, br_color);
				//gb_line(&gb, 0, item->height, item->width, item->height, br_color);
			}
			break;
		}
		case KMENUITEM_SEPARATOR:
		{
			gb_line(&gb, 
				item->left + item->padding_left, 
				item->padding_top, 
				item->left + item->width - item->padding_left - item->padding_right, 
				item->padding_top, 
				color_table->_3d_dark
			);
			gb_line(&gb, 
				item->left + item->padding_left, 
				item->padding_top+1, 
				item->left + item->width - item->padding_left - item->padding_right, 
				item->padding_top+1, 
				color_table->_3d_light
			);
			break;
		}
	}
	// put to wnd
	t_rect rc_draw = {0};
	rc_draw.left   = rc->left + item->margin_left;
	rc_draw.top    = rc->top + item->margin_top;
	rc_draw.right  = rc_draw.left + item->width;
	rc_draw.bottom = rc_draw.top + item->height;
	kos_PutRawImage(gb.bmp, &rc_draw);
	
	if (item->text)
	{
		t_label lbl = {0};
		lbl.font.color  = fg_color;
		lbl.font.type   = FONT_MONOSPACED | FONT_DRAW_ASCIIZ;

		lbl.position.left = rc_draw.left + item->margin_left + item->padding_left;
		lbl.position.top  = rc_draw.top + item->margin_top + item->padding_top;
		lbl.text = item->text;
		kos_PutLabel(&lbl);
	}
	
	if ((item->type == KMENUITEM_SUBMENU) && !(item->style & KMENUITEM_MAINMENU))
	{
		size_t font_width = 6;
		t_label lbl = {0};
		lbl.font.color  = fg_color;
		lbl.font.type   = FONT_MONOSPACED | FONT_DRAW_ASCIIZ;

		lbl.position.left = rc_draw.left + item->width -  item->margin_right - item->padding_right - font_width;
		lbl.position.top  = rc_draw.top + item->margin_top + item->padding_top;
		lbl.text = "\x10";
		kos_PutLabel(&lbl);
	}

}

kmenuitem_t * KMENU_API kmenuitem_new(int type, char *text, void *data)
{
	kmenuitem_t *item = malloc(sizeof(*item));
	if (!item)
		return NULL;
	
	char *textdup = "";
	if (text)
	{
		textdup = strdup(text);
		if (!textdup)
		{
			free(item);
			return NULL;
		}
	}

	memset(item, 0, sizeof(*item));
	item->type = type;
	item->text = textdup;
	item->is_enabled = 1;
	item->is_visible = 1;
	
	item->paint = kmenuitem_default_paint;
	
	size_t padding = font_height >> 1;
	item->padding_left = item->padding_top = item->padding_right = item->padding_bottom = padding;
	
	switch (type)
	{
		case KMENUITEM_NORMAL:
			//item->callback = (kmenuitem_callback_t)data;
			item->btnid  = (size_t)data;
			item->pref_height = font_height + item->padding_top + item->padding_bottom;
			item->pref_width  = strlen(text)*font_width + item->padding_left + item->padding_right;
			break;
		case KMENUITEM_SUBMENU:
			item->submenu = data;
			item->pref_height = font_height + item->padding_top + item->padding_bottom;
			item->pref_width  = strlen(text)*font_width  + item->padding_left + item->padding_right;
			break;
		case KMENUITEM_SEPARATOR:
			item->pref_height = KMENUITEM_SEPARATOR_HEIGHT + item->padding_top + item->padding_bottom;
			item->pref_width  = KMENUITEM_SEPARATOR_WIDTH + item->padding_left + item->padding_right;
			item->is_enabled = 0;
			break;
		default:
			free(item);
			return NULL;
	}
	item->width  = item->pref_width;
	item->height = item->pref_height;
	return item;
}

void KMENU_API kmenuitem_delete(kmenuitem_t *item)
{
	if (!item)
		return;

	if (item->text)
		free(item->text);

	free(item);
}

void KMENU_API kmenuitem_draw(kmenuitem_t *item, t_rect *rc)
{
	if (!item || !rc)
		return;
	
	item->paint(item, rc);
}


/*
void kmenuitem_dispatch_event(kmenuitem_t *item, int event)
{
	if (!item)
		return;
}
*/

size_t KMENU_API kmenuitem_get_preffered_width(kmenuitem_t *item)
{
	if (!item)
		return 0;
	return item->pref_width;
}

size_t KMENU_API kmenuitem_get_preffered_height(kmenuitem_t *item)
{
	if (!item)
		return 0;
	return item->pref_height;
}

typedef struct
{
	char *name;
	void *f;
} export_t;

export_t EXPORTS[] =
{
        { "START",       (void*)0x0 },
        { "version",     (void*)0x00010001 },
		{ "kmainmenu_draw",       kmainmenu_draw },
		{ "kmainmenu_dispatch_cursorevent",       kmainmenu_dispatch_cursorevent },
		{ "kmenu_init",       kmenu_init },
		
        { "ksubmenu_new",     ksubmenu_new    },
		{ "ksubmenu_delete",  ksubmenu_delete },
		{ "ksubmenu_draw",    ksubmenu_draw },
		{ "ksubmenu_add",     ksubmenu_add },
		
		{ "kmenuitem_new",    kmenuitem_new },
		{ "kmenuitem_delete", kmenuitem_delete },
		{ "kmenuitem_draw",   kmenuitem_draw },
		{ "kmenuitem_get_preffered_width", kmenuitem_get_preffered_width },
		{ "kmenuitem_get_preffered_height", kmenuitem_get_preffered_height },
        { NULL,          NULL },
};

#ifdef COMPILE_OBJ
#include "..\lib\system\api.c"
#include "..\lib\libc\string.c"
#include "..\lib\libc\stdlib.c"
#include "..\lib\libc\ctype.c"
#include "gblib.c"
#endif
