#ifndef _KOS_API_H
#define _KOS_API_H

#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>

#define MAKEDWORD(a,b) (((a) << 16) | ((b) & 0xFFFF))
#define RGB(r,g,b) ((((r) & 0xFF) << 16) | (((g) & 0xFF) << 8) | ((b) & 0xFF))

#define FONT_MONOSPACED  0x00000000
#define FONT_DEFAULT     0x10000000

#define FONT_DRAW_TO_BUFFER  0x08000000
#define FONT_DRAW_BACKGROUND 0x40000000
#define FONT_DRAW_ASCIIZ     0x80000000

#define BUTTON_DELETE 0x80000000
#define BUTTON_HIDDEN 0x40000000
#define BUTTON_WITHOUT_BORDER 0x20000000
#define BUTTONID_EXIT 1

#define SYSEVENT_WNDREDRAW 1
#define SYSEVENT_KEYPRESS 2
#define SYSEVENT_BTNPRESS 3
#define SYSEVENT_BKGREDRAW 5
#define SYSEVENT_MOUSE 6
#define SYSEVENT_IPC 7
#define SYSEVENT_NETWORK 8
#define SYSEVENT_DEBUG 9

#define THREAD_STACK_SIZE 4096

#define VK_RETURN 13
#define VK_ESC    27
#define VK_LEFT   176
#define VK_DOWN   177
#define VK_UP     178
#define VK_RIGHT  179

#pragma pack (push, 1)

typedef struct {
	uint8_t hours, minutes, seconds;
} t_time;


typedef struct {
	int type;
	uint32_t color;
	union {
		uint32_t bgcolor;
		void *buffer;
	};
} t_font;

typedef struct {
	uint16_t left, top;
} t_point;

typedef struct {
	t_point position;
	char *text;
	t_font font;
} t_label;

typedef struct {
	uint16_t left, top, right, bottom;
} t_rect;

typedef struct {
	uint32_t left, top, width, height;
} t_box;

typedef struct {
	t_rect size;
	uint32_t type;
	uint32_t id;
	uint32_t color;
} t_button;

typedef struct _t_dll_api {
	char *name;
	void *fn;
} t_dll_api;

typedef struct {
	uint32_t cpu_usage; // +0: dword: использование процессора (сколько тактов в секунду уходит на исполнение именно этого потока)
	uint16_t windows_stack_idx; //+4: word: позиция окна потока в оконном стэке
	uint16_t slot_thread; //+6: word: (не имеет отношения к запрошенному потоку)
	//номер слота потока, окно которого находится в оконном стэке
	//в позиции ecx
	uint16_t rezerved1; //+8: word: зарезервировано
	char process_name[12]; //+10 = +0xA: 11 байт: имя процесса
	//(имя запущенного файла - исполняемый файл без расширения)
	//+21 = +0x15: byte: зарезервировано, этот байт не изменяется
	uint32_t base_address; //+22 = +0x16: dword: адрес процесса в памяти
	uint32_t memory_usage; //+26 = +0x1A: dword: размер используемой памяти - 1
	uint32_t id; //+30 = +0x1E: dword: идентификатор (PID/TID)
	t_box window_size;
	//+34 = +0x22: dword: координата окна потока по оси x
	//+38 = +0x26: dword: координата окна потока по оси y
	//+42 = +0x2A: dword: размер окна потока по оси x
	//+46 = +0x2E: dword: размер окна потока по оси y
	uint16_t thread_state; //+50 = +0x32: word: состояние слота потока:
	/*0 = поток выполняется
	1 = поток приостановлен
	2 = поток приостановлен в момент ожидания события
	3 = поток завершается в результате вызова функции -1 или
	насильственно как следствие вызова подфункции 2 функции 18
	или завершения работы системы
	4 = поток завершается в результате исключения
	5 = поток ожидает события
	9 = запрошенный слот свободен, вся остальная информация о
	слоте не имеет смысла*/
	uint16_t rezerved2; //+52 = +0x34: word: зарезервировано, это слово не изменяется
	t_box client_area_size;
	/*+54 = +0x36: dword: координата начала клиентской области по оси x
	+58 = +0x3A: dword: координата начала клиентской области по оси y
	+62 = +0x3E: dword: ширина клиентской области
	+66 = +0x42: dword: высота клиентской области*/
	uint8_t window_state;
	/*+70 = +0x46: byte: состояние окна - битовое поле
	бит 0 (маска 1): окно максимизировано
	бит 1 (маска 2): окно минимизировано в панель задач
	бит 2 (маска 4): окно свёрнуто в заголовок*/
	uint32_t event_mask; //+71 = +0x47: dword: маска событий
	uint8_t keyboard_mode; //+75 = +0x4B: byte: режим ввода с клавиатуры(ASCII = 0; SCAN = 1)
} t_thread_info;

typedef struct {
	unsigned func;
	union {
		unsigned offset_lo;
		unsigned index;
		unsigned size_lo;
		unsigned flags;
	};
	union {
		unsigned offset_hi;
		unsigned flags2;
		unsigned size_hi;
		char *arg;
	};
	unsigned length;
	void *buffer;
	union {
		char fname[260];
		struct {
			char fname0;
			char *lp_fname;
		};
	};
} t_iofile;

typedef struct {
	uint32_t menu_body; // цвет фона системного меню
	uint32_t _3d_face; // цвет фона для придания 3D эффекта
	uint32_t _3d_dark; // цвет темной обводки для придания 3D эффекта
	uint32_t _3d_light; // цвет светлой обводки для придания 3D эффекта
	uint32_t win_title; // цвет текста в заголовке окна приложения
	uint32_t win_body; // цвет фона окна
	uint32_t btn_face; // цвет активной кнопки
	uint32_t btn_text; // цвет текста активной кнопки
	uint32_t win_text; // цвет текста в рабочей области окна
	uint32_t panel_frame; // цвет рамки области панели
	uint32_t win_face; // цвет бордюра окна между внешней и внутренней рамкой
	uint32_t win_inface; // цвет бордюра "неактивного" окна между внешней и внутренней рамкой
	uint32_t win_frame; // цвет внешней рамки окна
	uint32_t win_inframe; // цвет внешней рамки "неактивного" окна
	uint32_t win_border; // цвет внутренней рамки окна
	uint32_t win_inborder; // цвет внутренней рамки "неактивного" окна
	uint32_t win_graytext; // цвет в оттенках серого в рабочей области окна.
	uint32_t menu_frame; // цвет рамки системного меню
	uint32_t menu_text; // цвет текста системного меню
	uint32_t panel_body; // цвет фона области панели
	uint32_t panel_text; // цвет текста области панели
	uint32_t hint_frame; // цвет рамки всплывающей подсказки
	uint32_t hint_body; // цвет фона всплывающей подсказки
	uint32_t hint_text; // цвет текста всплывающей подсказки
	uint32_t btn_inface; // цвет фона "неактивной" кнопки
	uint32_t btn_fcface; // цвет фона кнопки при наведении курсора (focus)
	uint32_t btn_frame; // цвет рамки кнопки
	uint32_t btn_inframe; // цвет рамки "неактивной" кнопки
	uint32_t btn_fcframe; // цвет рамки кнопки при наведении курсора (focus)
	uint32_t btn_intext; // цвет текста "неактивной" кнопки
	uint32_t btn_fctext; // цвет текста кнопки при наведении курсора (focus)
	uint32_t gui_shadow; // цвет тени от любого текста
	uint32_t gui_face; // цвет фона элемента (edit,listbox,checkbox...)
	uint32_t gui_inface; // цвет фона "неактивного" элемента
	uint32_t gui_fcface; // цвет фона элемента при наведении курсора
	uint32_t gui_frame; // цвет рамки элемента
	uint32_t gui_inframe; // цвет рамки "неактивного" элемента
	uint32_t gui_fcframe; // цвет рамки элемента при наведении курсора
	uint32_t gui_text; // цвет текста элемента
	uint32_t gui_intext; // цвет текста "неактивного" элемента
	uint32_t gui_fctext; // цвет текста элемента при наведении курсора
	uint32_t gui_select; // цвет выделения текста
	uint32_t res_var_a; // зарезервированно
	uint32_t res_var_b; // зарезервированно
	uint32_t res_var_c; // зарезервированно
	uint32_t res_var_d; // зарезервированно
	uint32_t res_var_e; // зарезервированно
	uint32_t res_var_f; // зарезервированно
} t_system_colors;

#pragma pack(4)
typedef struct {
	unsigned version;
	unsigned length;
	unsigned count_of_files;
	char unknown[20];
} t_bdfk_header;

typedef struct {
	unsigned attrib;
	char fname_type;
	union {
		unsigned stamp;
		struct {
			char s;
			char m;
			char h;
		};
	} creation_time;
	union {
		unsigned stamp;
		struct {
			char d;
			char m;
			short y;
		};
	} creation_date;
	union {
		unsigned stamp;
		struct {
			char s;
			char m;
			char h;
		};
	} access_time;
	union {
		unsigned stamp;
		struct {
			char d;
			char m;
			short y;
		};
	} access_date;
	union {
		unsigned stamp;
		struct {
			char s;
			char m;
			char h;
		};
	} modification_time;
	union {
		unsigned stamp;
		struct {
			char d;
			char m;
			short y;
		};
	} modification_date;
	unsigned long long size;
	char fname[264];
} t_file_bdfk;

typedef struct {
	char find_path[264];
	char find_mask[264];
	t_bdfk_header header;
	t_file_bdfk bdfk;
	//char fname[264];
} t_findfileA;

#pragma pack (pop)

#define BDFK_FILE_ATTRIBUTE_READ_ONLY 1
#define BDFK_FILE_ATTRIBUTE_HIDDEN 0x2
#define BDFK_FILE_ATTRIBUTE_SYSTEM 0x4
#define BDFK_FILE_ATTRIBUTE_LABEL 0x8
#define BDFK_FILE_ATTRIBUTE_DIRECTORY 0x10
#define BDFK_FILE_ATTRIBUTE_NOT_ARCHIVED 0x20


extern int errno;

void kos_WindowRedrawStatus(unsigned int);
void kos_WindowRedrawStart(void);
void kos_WindowRedrawFinish(void);
void kos_DefineAndDrawWindow(unsigned x, unsigned y, unsigned w, unsigned h, unsigned styles, unsigned styles_ex, char *title);
void kos_WriteText(unsigned x, unsigned y, unsigned fncolor, unsigned bgcolor, unsigned style, char* str, unsigned len);
int kos_WaitForEvent(void);
size_t kos_SetEventMask(size_t mask);
int kos_GetKey(unsigned char* ch);
int kos_GetPressedButtonId(void);
int kos_CreateThread(void *proc, void *data);
void kos_ThreadYield(void);
int kos_GetCurrentThreadId(void);
void kos_ExitApp(void);
int kos_SendMessage(size_t event, size_t data);
int kos_GetSlotByTID(int tid);
void kos_TerminateThreadBySlot(int slot);
void kos_TerminateThreadById(int id);
int kos_GetForegroundWindow(void);
void kos_SetForegroundWindow(int slot);
t_thread_info *kos_GetThreadInfo(int thread_slot);
void kos_GetWindowSize(int thread_slot, t_rect *rc);
void kos_PutPixel(short x, short y, unsigned color);
void kos_PutLabel(t_label *label);
void kos_PutRawImage(void *img, t_rect *rc);
void kos_PutButton(t_button *btn);
uint16_t kos_GetFreeButtonId(void);
void kos_DeleteButton(t_button *btn);
void kos_Sleep(unsigned usec);
void kos_GetScreenSize(unsigned *x, unsigned *y);
void kos_DrawRectangle(unsigned x, unsigned y, unsigned w, unsigned h, unsigned color);
void kos_DrawLine(unsigned x, unsigned y, unsigned xto, unsigned yto, unsigned color);
//void kos_MouseGetPosByScreen(unsigned *x, unsigned *y);
//void kos_MouseGetPosByWindow(unsigned *x, unsigned *y);
void kos_GetCursorPosByScreen(t_point *pos);
void kos_GetCursorPosByWindow(t_point *pos);
size_t kos_GetMouseKeyState(void);

void kos_SystemStyleUpdate(void);
void kos_SetButtonStyle(int style);
void kos_SetSystemColors(t_system_colors* clrs);
void kos_GetSystemColors(t_system_colors* clrs);
int kos_GetHeightOfSkinHeader(void);

t_dll_api * kos_LoadLibrary(char *dllname);
void * kos_GetProcAddress(t_dll_api *fnlist, char *fnname);

size_t kos_HeapInit(void);
void *kos_HeapAlloc(size_t len);
int kos_HeapFree(void *mem);
void *kos_HeapRealloc(void *mem, size_t len);

int kos_ReadFile(char *fname, size_t offset, void *buf, size_t len);
int kos_ReadFileEx(t_iofile *f, size_t offset, void *buf, size_t len);
int kos_CreateFile(char *fname, size_t offset, void *buf, size_t len);
t_iofile * kos_CreateFileEx(char *fname);
int kos_WriteFile(char *fname, unsigned long long offset, void *buf, size_t len);
int kos_TruncateFile(char *fname, unsigned long long size);
int kos_GetFileInfo(char *fname, t_file_bdfk *fileinfo);
int kos_ExecFile(char *fname, char *arg, unsigned flags);
int kos_DeleteFile(char *fname);
int kos_MakeDir(char *fname);

void kos_SetCurrentDir(char *dirname);
void kos_GetCurrentDir(char *dirname, size_t len);

t_iofile *kos_FindFisrtFileA(char *path, t_findfileA **find);
bool kos_FindNextFileA(t_iofile *io, t_findfileA **find);

void kos_IPCInit(void *buffer, size_t size);
int kos_IPCSend(int tid, void *buffer, size_t size);

#endif
