#ifndef __LIBIO_H
#define __LIBIO_H

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#include <system/api.h>

#define O_BINARY 0x00
#define O_READ   0x01
#define O_WRITE  0x02
#define O_CREATE 0x04
#define O_SHARE  0x08
#define O_TEXT   0x10
 
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
 
typedef struct {
  union {
    uint32_t time;
    struct {
      uint8_t sec;
      uint8_t min;
      uint8_t hour;
    };
  };
  union {
    uint32_t date;
    struct {
      uint8_t day;
      uint8_t month;
      uint16_t year;
    };
  };
} FileDateTime;
 
typedef struct {
	uint32_t Function;
	uint32_t Position;
	uint32_t Flags;
	uint32_t Count;
	void *Buffer;
	union {
		char FileName[260];
		struct {
			char fname0;
			char *lpFileName;
		};
	};
} FileInfoBlock;
 
typedef struct {
	uint32_t Version;
	uint32_t FilesRead;
	uint32_t FilesCount;
	char unknown[20];
} FileInfoHeader;
 
typedef struct {
  uint32_t Attributes;
  uint32_t Flags;
  FileDateTime DateCreate;
  FileDateTime DateAccess;
  FileDateTime DateModify;
  union {
    uint64_t FileSize;
    struct {
      uint32_t FileSizeLow;
      uint32_t FileSizeHigh;
    };
  };
  char FileName[264];
} FileInfoA;
 
typedef struct {
  uint32_t Attributes;
  uint32_t Flags;
  FileDateTime DateCreate;
  FileDateTime DateAccess;
  FileDateTime DateModify;
  union {
    uint64_t FileSize;
    struct {
      uint32_t FileSizeLow;
      uint32_t FileSizeHigh;
    };
  };
  wchar_t FileName[264];
} FileInfoW;

typedef struct {
	void __stdcall (*init)(void);
	unsigned version;
	FileInfoA * __stdcall (*find_first)(char *path, char *fname, unsigned flag);
	FileInfoA * __stdcall (*find_next)(FileInfoA *handle);
	void __stdcall (*find_close)(FileInfoA *handle);
	int __stdcall (*size)(char *fname);
	void *__stdcall (*open)(char *fname, unsigned mode);
	int __stdcall (*read)(void *handle, void *buf, unsigned size);
	int __stdcall (*write)(void *handle, void *buf, unsigned size);
	int __stdcall (*seek)(void *handle, unsigned offset,unsigned base);
	unsigned __stdcall (*tell)(void *handle);
	bool __stdcall (*iseof)(void *handle);
	int __stdcall (*seteof)(void *handle);
	int __stdcall (*truncate)(void *handle);
	void __stdcall (*close)(void *handle);
} t_libio;

extern t_libio libio;

bool LoadLibIO(void);
 
#define FA_READONLY 0x01
#define FA_HIDDEN   0x02
#define FA_SYSTEM   0x04
#define FA_LABEL    0x08
#define FA_FOLDER   0x10
#define FA_ARCHIVED 0x20
#define FA_ANY      0x7F

#endif //__LIBIO_H
