#ifndef __LIBHTTP_H
#define __LIBHTTP_H

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

// status
#define FLAG_HTTP11             (1 << 0)
#define FLAG_GOT_HEADER         (1 << 1)
#define FLAG_GOT_ALL_DATA       (1 << 2)
#define FLAG_CONTENT_LENGTH     (1 << 3)
#define FLAG_CHUNKED            (1 << 4)
#define FLAG_CONNECTED          (1 << 5)

// user options
#define FLAG_KEEPALIVE          (1 << 8)
#define FLAG_STREAM             (1 << 9)
#define FLAG_REUSE_BUFFER       (1 << 10)

// error
#define FLAG_INVALID_HEADER     (1 << 16)
#define FLAG_NO_RAM             (1 << 17)
#define FLAG_SOCKET_ERROR       (1 << 18)
#define FLAG_TIMEOUT_ERROR      (1 << 19)
#define FLAG_TRANSFER_FAILED    (1 << 20)

typedef struct {
	uint32_t socket;               // socket on which the actual transfer happens
	uint32_t flags;                // flags, reflects status of the transfer using bitflags
	void *write_ptr;               // internal use only (where to write new data in buffer)
	size_t buffer_length;          // internal use only (number of available bytes in buffer)
	void *chunk_ptr;               // internal use only (where the next chunk begins)
	uint32_t timestamp;            // internal use only (when last data was received)

	uint32_t status;               // HTTP status
	size_t header_length;          // length of HTTP header
	void *content_ptr;             // ptr to content
	size_t content_length;         // total length of HTTP content
	size_t content_received;       // number of currently received content bytes

	char http_header[0];

} t_http_msg;

typedef struct {
	void __stdcall (*init)(void);
	unsigned version;
	void __stdcall (*get)(void);
	void __stdcall (*head)(void);
	void __stdcall (*post)(void);
	void __stdcall (*find_header_field)(void);
	void __stdcall (*process)(void);
	void __stdcall (*send)(void);
	void __stdcall (*receive)(void);
	void __stdcall (*disconnect)(void);
	void __stdcall (*free)(void);
	void __stdcall (*stop)(void);
	void __stdcall (*escape)(void);
	void __stdcall (*unescape)(void);
} t_libhttp;

extern t_libhttp libhttp;

bool LoadLibHTTP(void);

#endif //__LIBHTTP_H
