#include <byte_order.h>
#include <stdint.h>
#include <string.h>

#include <system/syscall.h>
#include <system/api.h>
#include <system/net/socket.h>

SOCKET socket(int af, int type, int protocol)
{
	return syscall5(75, 0, af, type, protocol);
}

int closesocket(SOCKET s)
{
	return syscall3(75, 1, s);
}

int bind(SOCKET s, const struct sockaddr *name, int namelen)
{
	return syscall5(75, 2, s, (unsigned)name, namelen);
}

int listen(SOCKET s, int backlog)
{
	return syscall4(75, 3, s, backlog);
}

int connect(SOCKET s, const struct sockaddr *name, int namelen)
{
	return syscall5(75, 4, s, (unsigned)name, namelen);
}

SOCKET accept(SOCKET s, struct sockaddr *addr, int *addrlen)
{
	return syscall5(75, 5, s, (unsigned)addr, (unsigned)addrlen);
}

int send(SOCKET s, const char *buf, int len, int flags)
{
	return syscall6(75, 6, s, (unsigned)buf, len, flags);
}

int recv(SOCKET s, char *buf, int len, int flags)
{
	return syscall6(75, 7, s, (unsigned)buf, len, flags);
}

int setsockopt(SOCKET s, int level, int optname, const char *optval, int optlen)
{
	optstruct *opt = kos_HeapAlloc(sizeof(*opt) + optlen);
	if (!opt)
		return -1;

	opt->level   = level;
	opt->optname = optname;
	opt->optlen  = optlen;
	
	memcpy(opt->optval, optval, optlen);
	
	int retval = syscall4(75, 8, s, (unsigned)opt);
	kos_HeapFree(opt);
	return retval;
}

int getsockopt(SOCKET s, int level, int optname, char *optval, int *optlen)
{
	optstruct *opt = kos_HeapAlloc(sizeof(*opt) + *optlen);
	if (!opt)
		return -1;

	opt->level   = level;
	opt->optname = optname;

	int retval = syscall4(75, 9, s, (unsigned)opt);

	*optlen = opt->optlen;	
	memcpy(optval, opt->optval, opt->optlen);
	
	kos_HeapFree(opt);	
	return retval;
}

uint32_t htonl(uint32_t x)
{
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned char *s = (u_char *)&x;
	return (uint32_t)(s[0] << 24 | s[1] << 16 | s[2] << 8 | s[3]);
#else
	return x;
#endif
}

uint16_t htons(uint16_t x)
{
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned char *s = (u_char *) &x;
	return (uint16_t)(s[0] << 8 | s[1]);
#else
	return x;
#endif
}

uint32_t ntohl(uint32_t x)
{
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned char *s = (void *)&x;
	return (uint32_t)(s[0] << 24 | s[1] << 16 | s[2] << 8 | s[3]);
#else
	return x;
#endif
}

uint16_t ntohs(uint16_t x)
{
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned char *s = (u_char *) &x;
	return (uint16_t)(s[0] << 8 | s[1]);
#else
	return x;
#endif
}
